#include <stdint.h>
#include <SDL2/SDL.h>

extern int clockSteps;
extern float displayScale;
extern uint8_t halt;
extern uint8_t showDebug;
extern uint8_t showHelp;

extern void resetSystem();
extern void irq6502();
extern void nmi6502();
extern void step6502();
extern void updateRenderStates();
extern void scanKeyboard();

static SDL_Scancode currentKey = SDL_SCANCODE_UNKNOWN;
static SDL_Event event;

void pollEvents(){
	while (SDL_PollEvent(&event)){
		switch(event.type){
			case SDL_QUIT:
				printf("Exited.\n");
				exit(0);
				
			case SDL_KEYDOWN:
				switch (event.key.keysym.scancode){
					case SDL_SCANCODE_F1:	/* Close Window or show help*/
						exit(EXIT_SUCCESS);
						break;
						
					case SDL_SCANCODE_F2:	/* Show Key Help */
						showHelp = !showHelp;
						break;
						
					case SDL_SCANCODE_F3:	/* Show Debug Menu */
						showDebug = !showDebug;
						break;
						
					case SDL_SCANCODE_F4:	/* CPU Halt */
						halt = !halt;
						break;
						
					case SDL_SCANCODE_F5:	/* CPU Reset */
						resetSystem();
						break;
						
					case SDL_SCANCODE_F6:	/* Scale -- */
						if (displayScale > 1) displayScale--;
					break;
					
					case SDL_SCANCODE_F7:	/* Scale ++ */
						displayScale++;
						break;
						
					case SDL_SCANCODE_F8:	/* CPU Singlestep */
						for (int i = 0; i < clockSteps; i++) step6502();
						break;
					
					case SDL_SCANCODE_F9:	/* CPU Save Snapshot */
						break;
						
					case SDL_SCANCODE_F10:	/* CPU Load recent or given snapshot */
						break;
						
					case SDL_SCANCODE_F11:	/* CPU NMI */
						nmi6502();
						break;
						
					case SDL_SCANCODE_F12:	/* CPU IRQ */
						irq6502();
						break;
						
					default:
						currentKey = event.key.keysym.scancode;
						break;
				}
				break;
			case SDL_KEYUP:
				// To tell keyboard.c, that no key is being pressed
				currentKey = SDL_SCANCODE_UNKNOWN;
			break;
			default:
				break;
		}
	}
}
