#include <stdlib.h>
#include <stdint.h>

extern float displayScale;

/* Public Variables */
int8_t videoModified;

/* Hardware-like Registers */
uint8_t videoX;
uint8_t videoY;
uint8_t videoMode;
uint8_t videoA;

/* Private Variables */
static uint8_t videoRow;
static uint8_t videoValue;
static uint8_t videoColorValue;

static uint32_t videoColorIndex[] = {
	0x000000FF,
	0xFF0000FF,
	0x00FF00FF,
	0xFFFF00FF,
	0x0000FFFF,
	0xFF00FFFF,
	0x00FFFFFF,
	0xFFFFFFFF
};
static uint8_t videoMemory[0x8000];
static uint32_t renderMemory[(SDL_X_SIZE * SDL_Y_SIZE)];


void writeVideo(){
	videoMemory[videoX + (videoY * 128)] = videoA;
	videoModified = 1;
}

uint8_t readVideo(){
	return videoMemory[videoX + (videoY * 128)];
}

void initVideo(){
	videoModified = 1;
	videoMode = 0x01;
	
	for (int i = 0; i < 0x8000; i++) videoMemory[i] = rand();
}

void updateVideo(){
	if (videoModified){
		//sfVertexArray_clear(renderArray);
		
		for (uint8_t y = 0; y < 255; y++){
			videoRow = (int8_t)(y & 0x7);
			
			for (uint8_t x = 0; x < 48; x++){
				
				if ((videoMode & 0x01) == 0){
					/*  
					*           Text Address Format
					*           0bRRR1111AAAAAAAA
					* 
					*           R = row pins
					*           A = address
					*/
					
					// Get text index value
					videoValue = videoMemory[(((y & 0xF8) >> 3) << 7) | x];
					
					// Index the value in font location
					videoValue = videoMemory[(videoRow | 0b1111000) + (videoValue * 128)];
					videoColorValue = videoMemory[(((y & 0xF8) >> 3) << 7) + (x | 0x40)];
				}
				else {
					videoValue = videoMemory[(y * 128) + x];
					videoColorValue = videoMemory[(y * 128) + (x | 0x40)];
				}
				
				// Snipping out the upper 8 colors
				videoColorValue &= 0x77;
				
				for (uint8_t i = 0; i < 8; i++){
					if (displayScale == 1){
						renderMemory[(y * SDL_X_SIZE) + ((x * 8) + i)] = (videoValue & (128 >> i)) ? videoColorIndex[((videoColorValue & 0xF0) >> 4)] : videoColorIndex[(videoColorValue & 0x0F)];
					} else {
						for (uint8_t _x = 0; _x < displayScale; _x++){
							for (uint8_t _y = 0; _y < displayScale; _y++){
								renderMemory[(((y * (int)displayScale) + _y) * SDL_X_SIZE) + ((((x * (int)displayScale) * 8) + (i * (int)displayScale)) + _x)] = (videoValue & (128 >> i)) ? videoColorIndex[((videoColorValue & 0xF0) >> 4)] : videoColorIndex[(videoColorValue & 0x0F)];
							}
						}
					}
				}
			}
		}
		
		videoModified = 0;
	}
}



